/** 
 *   Site testing for SKA candidate.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/


#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <sys/file.h>
#include <sys/stat.h>

#include "my_stdio.h"
#include "observ_table.h"
#include "analyzer.h"



static int is_mode(char mode)
{
	int res = -1;

	switch(mode) {
	case '0':
	case 'c':
	case 'C': res = 0; break;
	case '1': res = 1; break;
	case '2': res = 2; break;
	default:  SYSLOG(LOG_ERR,"Unknow mode %c, ignored.", mode);
	}

	return res;
}

static struct modes *make_modes(void)
{
	struct modes *obs;
	
	obs = (struct modes *) malloc(sizeof(struct modes));
	if( obs == NULL ){
		PVERB(PERR,"make_modes: error alocating struct modes.\n");
		return NULL;
	}
	obs->next = (struct modes *) NULL;
	obs->mode = -1;

	return obs;
}


static struct sequence *make_sequence(void)
{
	struct sequence *seq;
	
	seq = (struct sequence *) malloc(sizeof(struct sequence));
	if( seq == NULL ){
		PVERB(PERR,"make_sequence: error alocating struct sequence.\n");
		return NULL;
	}
	seq->type = SEQ_TYPE_NULL;
	seq->next = (struct sequence *) NULL;
	seq->mode = -1;
	seq->level = 0;
	seq->loop = 0;
	seq->azimuth = -1;
	seq->polarization = '?';
	seq->partition = 1;

	return seq;
}


static int get_modes(char *s, struct modes **o)
{
	int  res;
	char mode;
	struct modes *op = *o;

	res = sscanf(s,"%c %lf %lf %lf %d %d %c %lf %lf %lf",
		     &mode, &op->start_freq, &op->stop_freq, 
		     &op->RBW, &op->sweep, &op->repetitions, &op->switchs,
		     &op->referance_level, &op->VBW, &op->disp_escale);
	switch(res) {
	case 7:
		op->referance_level = AN_REF;
	case 8:
		op->VBW = -1; /* auto mode */
	case 9:
		op->disp_escale = AN_LG;
	case 10:
		break;
	default: 
		SYSLOG(LOG_ERR,
		       "get_modes: Error in modes file (get %d)", res);
		return -1;
		break;
	}
	op->mode = is_mode(mode);
	if (op->mode < 0) 
		return -1;

	return 0;
}


struct modes *get_modes_table(char *file_name)
{
	FILE *fp;
	char line[160];
	int start = 0;
	int i;
	struct modes *obs, *obsp,
		*obsl = NULL;

	obsp = obs = make_modes();
	fp = fopen(file_name, "r");
	if( fp == NULL ){
		SYSLOG(LOG_ERR,
		       "get_modes_table: Error opening file: %s",file_name);
		free(obsp);
		return NULL;
	}
	while( fgets(line, sizeof(line), fp)){
		if( memcmp(stripwhite(line),"observ",6) == 0 && start){
			for( i=0; (i < 160) && (line[i] != '=');i++);
			i++;
			if( get_modes(stripwhite(line+i), &obsp) == 0 ){
				obsl = obsp;
				if(obsp == NULL) {
					SYSLOG(LOG_ERR,
					       "get_modes_table: obsp is NULL, exiting now...");
					exit(1);
				}
				obsp->next = make_modes();
				obsp = obsp->next;
				obsp->next = NULL;
			}
		}else{
			if( memcmp(stripwhite(line),"start",5) == 0 ) 
				start = 1;
			if( memcmp(stripwhite(line),"stop",4) == 0 )  
				start = 0;
		}
	}
	fclose(fp);

	if ( obsl ) {
		free_modes_table(obsp);
		obsl->next = NULL;
	}

	return obs;
}


int free_modes_table(struct modes *obs)
{
	struct modes *op = obs;
	struct modes *opp;

	while(op != NULL){
		opp = op;
		op = op->next;
		free(opp);
	}

	return 0;
}


int free_sequence_table(struct sequence *seq)
{
	struct sequence *sp = seq;
	struct sequence *spp;

	while(sp != NULL){
		spp = sp;
		sp = sp->next;
		free(spp);
	}

	return 0;
}

static int get_sequence(char *s, struct sequence **o)
{
	int res;
	char mode;
	struct sequence *sp = *o;

	res = sscanf(s,"%c %d %d %c",
		     &mode, &sp->partition, &sp->azimuth, &sp->polarization);
	if(res != 4){
		SYSLOG(LOG_ERR,
		       "get_sequence: Error in sequence file (get %d)", res);
		return -1;
	}      
	sp->mode = is_mode(mode);
	if (sp->mode < 0) return -1;

	return 0;
}



struct sequence *get_sequence_table(char *file_name)
{
	FILE *fp;
	char line[160];
	char *linep;
	int start = 0;
	int i;
	struct sequence *seq, *seqp;
	int level = 0;
	int loop_count[SEQ_MAX_LOOP];

	loop_count[level] = 1;
	seqp = seq = make_sequence();
	fp = fopen(file_name, "r");
	if( fp == NULL ){
		SYSLOG(LOG_ERR,
		       "get_sequence_table: Error opening file: %s",
		       file_name);
		free(seqp);
		return NULL;
	}
	while( fgets(line, sizeof(line), fp)){
		linep = stripwhite(line);
		if( memcmp(linep,"seq",3) == 0 && start){
			for( i=0; (i < 160) && (line[i] != '=');i++);
			i++;
			if( get_sequence(stripwhite(line+i), &seqp) == 0 ){
				if(seqp == NULL) {
					SYSLOG(LOG_ERR,
					       "get_sequence_table: obsp is NULL, exiting now...");
					exit(1);
				}
				seqp->type = SEQ_TYPE_DATA;
				seqp->level = level;
				seqp->loop = loop_count[level];
				seqp->next = make_sequence();
				seqp = seqp->next;
			}
			else{
				printf("## EROR: %s", line);
			}
		}		
		if( memcmp(linep,"start", 5) == 0 ) 
			start = 1;
		if( memcmp(linep,"stop",4) == 0 )  
			start = 0;
		if( memcmp(linep,"restart",7) == 0 ) {
			seqp->type = SEQ_TYPE_RESTART;
			seqp->level = level;
			seqp->loop = loop_count[level];	
			start = 0;
			seqp->next = make_sequence();
			seqp = seqp->next;
		}
		if(memcmp(linep,"loop",4) == 0 && start){
			seqp->type = SEQ_TYPE_LOOP;
			seqp->level = level;
			seqp->loop = loop_count[level];	
			seqp->next = make_sequence();
			seqp = seqp->next;
			level++;
			if(level == SEQ_MAX_LOOP) {
				SYSLOG(LOG_ERR,
				       "get_sequence_table: loop level exeded, exiting now...");
				exit(1);
			}else{
				loop_count[level] = 
					(int) strtol(linep+5,NULL,10);
			}
		}
		if(memcmp(linep,"pool",4) == 0 && start){
			if(level) level--;
			seqp->type = SEQ_TYPE_POOL;
			seqp->level = level;
			seqp->loop = loop_count[level];	
			seqp->next = make_sequence();
			seqp = seqp->next;
		}
	}
	fclose(fp);

	return seq;
}


int time_estimation_mode(int mode, struct modes *mod, int partition, 
			    int *count, time_t *time_est)
{
	struct modes *modp = mod;
	double est = 0.0;
	int split;
	int sweep;
	
	*count = 0;

	while (modp != NULL) {
		split = 1;
		if (modp->mode == mode) {

			sweep = (int) ceil((double) modp->sweep / 
					   (double) partition);
			/*
			printf("## %3d %3d %3d\n", modp->sweep, sweep,
			       modp->repetitions);
			*/
			if (sweep > SWEEP_MAX){
				split = (int) ceil((double) sweep / 
						   (double) SWEEP_MAX);
				sweep = (int) ceil((double) sweep / 
						   (double) split);
			}
			modp->sweep_obs = sweep * split;
			est += ((double) modp->sweep_obs + 
				(SOME_LOST_TIME * (double) split)) *
				(double)modp->repetitions;
			*count += modp->repetitions;
		}
		modp = modp->next;
	}
		
	*time_est = (time_t) est;
	return 0;
}


